﻿local L = LibStub("AceLocale-3.0"):GetLocale("SoDWhisper")
local AceConfig = LibStub("AceConfig-3.0")
local C = LibStub("AceConfigDialog-3.0")
local optFrame
local backgrounds, borders, fonts, sounds = {},{},{},{}

--local constants
local TIME_FORMATS = {
	["[%X]"] = L["HH:MM:SS (24-hour)"],
	["[%H:%M]"] = L["HH:MM (24-hour)"],
	["[%I:%M:%S %p]"] = L["HH:MM:SS AM (12-hour)"],
	["[%I:%M:%S]"] = L["HH:MM:SS (12-hour)"],
	["[%I:%M %p]"] = L["HH:MM AM (12-hour)"],
	["[%I:%M]"] = L["HH:MM (12-hour)"],
	["[%M:%S]"] = L["MM:SS"],
	[""] = L["None"],
}

local SOUND_OUTPUT = {
	["Master"] = L["Master sound"],
	["SFX"] = L["SFX sound"],
	["Ambience"] = L["Ambience sound"],
	["Music"] = L["Music sound"],
}

local REALID_FORMATS = {
	["FIRST"] = L["First name"],
	["LAST"] = L["Last name"],
	["FULL"] = L["Full name"],
}

-- generic options
local options = {
	type = "group",
	name = "SoDWhisper",
	args = {
		general = {
			type = "group",
			name = L["General"],
			args = {
				hints = {
					type = "group",
					order = 5,
					name = L["Hints"],
					childGroups = "tab",
					args = {
						editboxhints = {
							type = "group",
							order = 5,
							name = L["Editbox Plugin"],
							args = {
								editboxdescriptionCT = {
									type = "group",
									order = 5,
									name = L["Player Title"],
									inline = true,
									args = {
										editboxdescription = {
											type = "description",
											order = 5,
											name = "|c00ffff00"..L["Page Icon"].."|r - |c001eff00"..L["Open Chat History Log Frame for player"]..
												"|r\n|c00ffff00"..L["Player Name"].."|r - |c001eff00"..L["Open Change Display Name Frame for player"]..
												"|r\n|c00ffff00"..L["Group Icon"].."|r - |c001eff00"..L["Invite player to group"]..
												"|r\n|c00ffff00"..L["Add Icon"].."|r - |c001eff00"..L["Add player to friends"]..
												"|r\n|c00ffff00"..L["Ignore Icon"].."|r - |c001eff00"..L["Ignore or stop ignoring player"]..
												"|r\n|c00ffff00"..L["Location Icon"].."|r - |c001eff00"..L["Display information about player"].."|r",
										},
									},
								},
							},
						},
						brokerhints = {
							type = "group",
							order = 10,
							name = L["LDB/Minimap Plugin"],
							args = {
								brokerplugindescriptionBI = {
									type = "group",
									order = 5,
									name = L["LDB/Minimap Icon"],
									inline = true,
									args = {
										brokerplugindescriptionBI = {
											type = "description",
											order = 5,
											name = "|c00ffff00"..L["Left Click"].."|r - |c001eff00"..L["Whisper player of missed message"]..
												"|r\n|c00ffff00"..L["Right Click"].."|r - |c001eff00"..L["Open options"]..
												--"|r\n|c00ffff00"..L["Ctrl"].." + "..L["Left Click"].."|r - |c001eff00"..L["Open friends frame"]..
												--"|r\n|c00ffff00"..L["Ctrl"].." + "..L["Right Click"].."|r - |c001eff00"..L["Open guild frame"]..
												"|r\n|c00ffff00"..L["Shift"].." + "..L["Ctrl"].." + "..L["Alt"].. " + "..L["Left Click"].."|r - |c001eff00"..L["DELETE ALL history"].."|r",
										},
									},
								},
								brokerplugindescriptionPN = {
									type = "group",
									order = 10,
									name = L["Player Names"],
									inline = true,
									args = {
										brokerplugindescriptionPN = {
											type = "description",
											order = 5,
											name = "|c00ffff00"..L["Left Click"].."|r - |c001eff00"..L["Reply to player"]..
												"|r\n|c00ffff00"..L["Right Click"].."|r - |c001eff00"..L["Reply to BattleNet character if applicable"]..
												"|r\n|c00ffff00"..L["Alt"].. " + "..L["Left Click"].."|r - |c001eff00"..L["Invite player to group"]..
												"|r\n|c00ffff00"..L["Ctrl"].." + "..L["Left Click"].."|r - |c001eff00"..L["Open Chat History Log Frame for player"]..
												"|r\n|c00ffff00"..L["Ctrl"].." + "..L["Right Click"].."|r - |c001eff00"..L["Open Change Display Name Frame for player"]..
												"|r\n|c00ffff00"..L["Shift"].." + "..L["Left Click"].."|r - |c001eff00"..L["DELETE player history"].."|r",
										},
									},
								},
								brokerplugindescriptionGT = {
									type = "group",
									order = 15,
									name = L["Group Title"],
									inline = true,
									args = {
										brokerplugindescriptionGT = {
											type = "description",
											order = 5,
											name = "|c00ffff00"..L["Left Click"].."|r - |c001eff00"..L["Toggle between Timeframe and Online"]..
												"|r\n|c00ffff00"..L["Middle Click"].."|r - |c001eff00"..L["Toggle showing BattleNet App players that are away"]..
												"|r\n|c00ffff00"..L["Shift"].." + "..L["Left Click"].."|r - |c001eff00"..L["DELETE history for group (ex. All of Guild)"].."|r",
										},
									},
								},
							},
						},
					},
				},
------------------------------------------------------------------------------------------------------------------------------------------------------------
				appearance = {
					type = "group",
					order = 10,
					name = L["Appearance"],
					args = {
						linesShown = {
							type = "range",
							order = 5,
							name = L["Entries Shown"],
							desc = L["The number of entries to show in the message panels."],
							min = 1,
							max = 15,
							step = 1,
							set = function(_,v) SoDWhisper.db.profile.linesShown=v end,
							get = function() return SoDWhisper.db.profile.linesShown end,
						},
						textures = {
							type = "group",
							order = 10,
							name = L["Texture"],
							inline = true,
							args = {
								color = {
									type = "color",
									order = 5,
									name = L["Background Color"],
									hasAlpha = true,
									get = function()
										local c = SoDWhisper.db.profile.paneColor
										return c[1], c[2], c[3], c[4]
									end,
									set = function(_,r, g, b, a)
										SoDWhisper.db.profile.paneColor = {r, g, b, a}
										SoDWhisper:SendMessage("SoDWhisper_SKIN")
									end,
								},
								background = {
									type = 'select',
									order = 10,
									name = L["Background"],
									desc = L["Change the background texture (For some textures the background color needs to be set to white)."],
									values = backgrounds,
									get = function() return SoDWhisper.db.profile.paneBackground end,
									set = function(_,v)
										SoDWhisper.db.profile.paneBackground = backgrounds[v]
										SoDWhisper:SendMessage("SoDWhisper_SKIN")
									end,
								},
								borderColor = {
									type = 'color',
									order = 15,
									name = L["Border Color"],
									hasAlpha = true,
									get = function()
										local c = SoDWhisper.db.profile.paneBorderColor
										return c[1], c[2], c[3], c[4]
									end,
									set = function(_,r, g, b, a)
										SoDWhisper.db.profile.paneBorderColor = {r, g, b, a}
										SoDWhisper:SendMessage("SoDWhisper_SKIN")
									end,
								},        
								border = {
									type = 'select',
									order = 20,
									name = L["Border Style"],
									desc = L["Change the border style of the panel."],
									values = borders,
									get = function() return SoDWhisper.db.profile.paneBorder end,
									set = function(_,v)
										SoDWhisper.db.profile.paneBorder = borders[v]
										SoDWhisper:SendMessage("SoDWhisper_SKIN")
									end,
								},
							},
						},
						text = {
							type = "group",
							order = 15,
							name = L["Text"],
							inline = true,
							args = {
								colorOut = {
									type = "color",
									order = 5,
									name = L["Color Outgoing"],
									desc = L["Set the color of outgoing messages."],
									get = function()
										local c = SoDWhisper.db.profile.colorOutgoing
										return c[1], c[2], c[3], c[4]
									end,
									set = function(_,r, g, b, a)
										SoDWhisper.db.profile.colorOutgoing = {r, g, b, a}
										SoDWhisper:SendMessage("SoDWhisper_SKIN")
									end,
								},
								colorIn = {
									type = "color",
									order = 10,
									name = L["Color Incoming"],
									desc = L["Set the color of incoming messages."],
									get = function()
										local c = SoDWhisper.db.profile.colorIncoming
										return c[1], c[2], c[3], c[4]
									end,
									set = function(_,r, g, b, a)
										SoDWhisper.db.profile.colorIncoming = {r, g, b, a}
										SoDWhisper:SendMessage("SoDWhisper_SKIN")
									end,
								},
								font = {
									type = 'select',
									order = 15,
									name = L["Font"],
									values = fonts,
									get = function() return SoDWhisper.db.profile.paneFont end,
									set = function(_,v) 
										SoDWhisper.db.profile.paneFont = fonts[v]
										SoDWhisper:SkinMyFrame(SoDWhisper.LogFrame, true)
										SoDWhisper:SkinMyFrame(SoDWhisper.createDisplayName, nil, true)
										SoDWhisper:SendMessage("SoDWhisper_SKIN")
									end,
								},
								fontsize = {
									type = "range",
									order = 20,
									name = L["Font Size"],
									min = 6,
									max = 20,
									step = 1,
									get = function() return SoDWhisper.db.profile.fontSize end,
									set = function(_,v) 
										SoDWhisper.db.profile.fontSize = v 
										SoDWhisper:SkinMyFrame(SoDWhisper.LogFrame, true)
										SoDWhisper:SkinMyFrame(SoDWhisper.createDisplayName, nil, true)
										SoDWhisper:SendMessage("SoDWhisper_SKIN")
									end,
								},
							},
						},
						timestamps = {
							type = "group",
							order = 20,
							name = L["Timestamp"],
							inline = true,
							args = {
								timeColor = {
									type = "color",
									order = 5,
									name = L["Timestamp Color"],
									get = function()
										local c = SoDWhisper.db.profile.timeColor
										return c[1], c[2], c[3], c[4]
									end,
									set = function(_,r, g, b, a)
										SoDWhisper.db.profile.timeColor = {r, g, b, a}
									end,
									disabled = function() return SoDWhisper.db.profile.timeFormat == "" end,
								},
								timeFormat = { 
									type = "select",
									order = 10,
									name = L["Timestamp Format"],
									values = TIME_FORMATS,
									get = function() return SoDWhisper.db.profile.timeFormat end,
									set = function(_, v)
									SoDWhisper.db.profile.timeFormat = v
									end,
								},
							},
						},
					},
				},
------------------------------------------------------------------------------------------------------------------------------------------------------------
				names = {
					type = "group",
					order = 15,
					name = L["Names"],
					args = {
						RealIDNames = { 
							type = "select",
							order = 5,
							name = L["RealID Display"],
							desc = L["Select the display for realID names."],
							values = REALID_FORMATS,
							get = function() return SoDWhisper.db.profile.realIDName end,
							set = function(_, v)
							SoDWhisper.db.profile.realIDName = v
							SoDWhisper:UpdateBNFriends(true)
							end,
						},
						colornames = {
							type = "group",
							order = 10,
							name = L["Color By Class"],
							inline = true,
							args = {
								classColor = {
									type = "toggle",
									order = 5,
									name = L["Character Names"],
									desc = L["Color character names by class."],
									set = function(_,v) 
										SoDWhisper.db.profile.classColor = v
									end,
									get = function() return SoDWhisper.db.profile.classColor end,
								},
								colorBN = {
									type = "toggle",
									order = 10,
									name = L["BattleNet Names"],
									desc = L["Color BattleNet names by character class when possible."],
									set = function(_,v) 
										SoDWhisper.db.profile.colorBN = v
									end,
									get = function() return SoDWhisper.db.profile.colorBN end,
								},
							},
						},
					},
				},
------------------------------------------------------------------------------------------------------------------------------------------------------------
				sound = {
					type = "group",
					order = 20,
					name = L["Sounds"],
					args = {
						useSound = {
							type = "toggle",
							order = 5,
							name = L["Enable"],
							desc = L["Use Sound for incoming messages."],
							get = function() return SoDWhisper.db.profile.useSound end,
							set = function(_,v) SoDWhisper.db.profile.useSound = v end,
						},
						channel = { 
							type = "select",
							order = 10,
							name = L["Channel Output"],
							desc = L["Channel for sound to play through."],
							values = SOUND_OUTPUT,
							get = function() return SoDWhisper.db.profile.soundchannel end,
							set = function(_, v) SoDWhisper.db.profile.soundchannel = v end,
							disabled = function() return not SoDWhisper.db.profile.useSound end,
						},
						CustomSound = {
							type = "group",
							order = 15,
							name = L["Custom Sounds"],
							inline = true,
							disabled = function() return not SoDWhisper.db.profile.useSound end,
							args = {
								Regwhisper = {
									type = 'select',
									order = 5,
									name = L["Other Whisper"],
									desc = L["Sound for whispers that are not from guild or friends."],
									values = sounds,
									get = function() return SoDWhisper.db.profile.Otherwhispersound end,
									set = function(_,v) 
										SoDWhisper.db.profile.Otherwhispersound = sounds[v]
										local sT = SoDWhisper.media:Fetch('sound', SoDWhisper.db.profile.Otherwhispersound)
										PlaySoundFile(sT, SoDWhisper.db.profile.soundchannel)
									end,
								},
								Guildwhisper = {
									type = 'select',
									order = 10,
									name = L["Guild Whisper"],
									desc = L["Sound for guild whispers."],
									values = sounds,
									get = function() return SoDWhisper.db.profile.Guildwhispersound end,
									set = function(_,v) 
										SoDWhisper.db.profile.Guildwhispersound = sounds[v]
										local sT = SoDWhisper.media:Fetch('sound', SoDWhisper.db.profile.Guildwhispersound)
										PlaySoundFile(sT, SoDWhisper.db.profile.soundchannel)
									end,
								},
								Friendwhisper = {
									type = 'select',
									order = 15,
									name = L["Friend Whisper"],
									desc = L["Sound for friend whispers."],
									values = sounds,
									get = function() return SoDWhisper.db.profile.Friendwhispersound end,
									set = function(_,v) 
										SoDWhisper.db.profile.Friendwhispersound = sounds[v]
										local sT = SoDWhisper.media:Fetch('sound', SoDWhisper.db.profile.Friendwhispersound)
										PlaySoundFile(sT, SoDWhisper.db.profile.soundchannel)
									end,
								},
							},
						},
					},
				},
------------------------------------------------------------------------------------------------------------------------------------------------------------
				history = {
					type = "group",
					order = 25,
					name = L["History"],
					args = {
						history = {
							type = "range",
							order = 1,
							name = L["Saved History"],
							desc = L["Timeframe in hours messages will be kept (When set to 0, messages will be kept only for the session)."],
							min = 0,
							max = 720,
							step = 1,
							get = function() return SoDWhisper.db.profile.timeout end,
							set = function(_,v) SoDWhisper.db.profile.timeout = v end,
						},
						clear = {
							type = "execute",
							order = 2,
							name = L["Clear"],
							desc = L["Clears player and message history."],
							confirm = true,
							confirmText = L["Are you sure you want to clear the history?"],
							func = function() 
							wipe(SoDWhisper.db.factionrealm.chatHistory) 
							wipe(SoDWhisper.db.global.chatHistory) 
							SoDWhisper:UpdateOnLoad()
							end,
						},
					},
				},
------------------------------------------------------------------------------------------------------------------------------------------------------------
				miscellaneous = {
					type = "group",
					order = 30,
					name = L["Miscellaneous"],
					args = {
						BTNotify = {
							type = "toggle",
							order = 5,
							name = L["BattleTag Notify"],
							desc = L["Automatic BattleTag notification to players that have never setup a BattleTag (Helps save their history)."],
							get = function() return SoDWhisper.db.profile.BTNotify end,
							set = function(_,v) SoDWhisper.db.profile.BTNotify = v end,
						},
						combatUpdate = { 
							type = "toggle",
							order = 10,
							name = L["Update In Combat"],
							desc = L["Update status in combat."],
							get = function() return SoDWhisper.db.profile.combatUpdate end,
							set = function(_, v) SoDWhisper.db.profile.combatUpdate = v end,
						},
						inputSticky = { 
							type = "toggle",
							order = 15,
							name = L["Whisper Sticky"],
							desc = L["Do NOT allow the Enter Key to open Whispers (Keeps missed message count correct)."],
							get = function() return SoDWhisper.db.profile.inputSticky end,
							set = function(_, v)
								SoDWhisper.db.profile.inputSticky = v
								if v then
									ChatTypeInfo["WHISPER"].sticky = 0
									ChatTypeInfo["BN_WHISPER"].sticky = 0
								else
									ChatTypeInfo["WHISPER"].sticky = 1
									ChatTypeInfo["BN_WHISPER"].sticky = 1
								end
							end,
						},
						ignoreDBM = { 
							type = "toggle",
							order = 20,
							name = L["Ignore DBM"],
							desc = L["Ignore DBM auto reply whispers."],
							get = function() return SoDWhisper.db.profile.ignoreDBM end,
							set = function(_, v) SoDWhisper.db.profile.ignoreDBM = v end,
						},
					},
				},
			},
		},
		modules = {
			type = "group",
			name = L["Modules"],
			args = {}
		},
	},
}

SoDWhisper.media:Register("sound", "SoDWhisper", "Interface\\AddOns\\SoDWhisper\\MediaFiles\\IncomingWhisper.ogg")
function SoDWhisper:LibSharedMedia_Registered()
	for k,v in pairs(SoDWhisper.media:List("background")) do
		backgrounds[v] = v
	end
	for k,v in pairs(SoDWhisper.media:List("border")) do
		borders[v] = v
	end
	for k,v in pairs(SoDWhisper.media:List("font")) do
		fonts[v] = v
	end
	for k,v in pairs(SoDWhisper.media:List("sound")) do
		sounds[v] = v
	end
end

function SoDWhisper:SetupOptions()
	for k, v in self:IterateModules() do
		options.args.modules.args[string.gsub(k, " ", "_")] = {
			type = "group",
			name = (v.modName or k),
			childGroups = "tab",
			args = nil
		}
		local t
		if v.GetOptions then
			t = v:GetOptions()
		end
		t = t or {}
		t.group = {
			type = "group",
			order = 1,
			name = v.modName or k,
			inline = true,
			args = {
				temptoggle = {
					type = "toggle", 
					order = 5,
					name = v.toggleLabel or L["Enable"], 
					desc = v.Info and v:Info() or (L["Enable"].." "..(v.modName or k)), 
					get = function()
						return SoDWhisper.db.profile.modules[k] ~= false or false
					end,
					set = function(_, v)
						SoDWhisper.db.profile.modules[k] = v
						if v then
							SoDWhisper:EnableModule(k)
						else
							SoDWhisper:DisableModule(k)
						end
					end
				},
			},
		}
		options.args.modules.args[string.gsub(k, " ", "_")].args = t
	end
	-- from Whisp
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	
	optFrame = C:AddToBlizOptions("SoDWhisper", "SoDWhisper")
	AceConfig:RegisterOptionsTable("SoDWhisper", options)
	self:RegisterChatCommand("SoDWhisper", SoDWhisper.OpenConfig)
	
	SoDWhisper.media.RegisterCallback(SoDWhisper, "LibSharedMedia_Registered")
	self:LibSharedMedia_Registered()
	-- Thanks
end

function SoDWhisper:OpenConfig(input)
	if not C.OpenFrames["SoDWhisper"] then 
		C:Open("SoDWhisper") 
	else 
		C:Close("SoDWhisper") 
	end
end